@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atimach64xv.c,v 1.6 2003/07/19 15:26:54 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atimach64xv.c,v 1.8 2004/01/05 16:42:03 tsi Exp $ */
 /*
- * Copyright 2003 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 2003 through 2004 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -1438,7 +1438,7 @@ ATIMach64XVInitialiseAdaptor
     pAdaptor->PutImage             = ATIMach64PutImage;
     pAdaptor->QueryImageAttributes = ATIMach64QueryImageAttributes;
 
-    REGION_INIT(pScreen, &pATI->VideoClip, NullBox, 0);
+    REGION_NULL(pScreen, &pATI->VideoClip);
     pATI->ActiveSurface = FALSE;
 
     if (ATIMach64XVAtomGeneration != serverGeneration)
