@@ -23,7 +23,7 @@
 /* Hacked together from mga driver and 3.3.4 NVIDIA driver by Jarno Paananen
    <jpaana@s2.org> */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nv/riva_driver.c,v 1.4 2003/10/30 17:37:09 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nv/riva_driver.c,v 1.6 2004/01/10 22:31:53 mvojkovi Exp $ */
 
 #include "riva_include.h"
 
@@ -485,7 +485,6 @@ RivaPreInit(ScrnInfoPtr pScrn, int flags)
     RivaPtr pRiva;
     MessageType from;
     int i;
-    int bytesPerPixel;
     ClockRangePtr clockRanges;
     const char *s;
 
@@ -610,8 +609,6 @@ RivaPreInit(ScrnInfoPtr pScrn, int flags)
 	}
     }
 
-    bytesPerPixel = pScrn->bitsPerPixel / 8;
-
     /* The vgahw module should be loaded here when needed */
     if (!xf86LoadSubModule(pScrn, "vgahw")) {
 	xf86FreeInt10(pRiva->pInt);
@@ -1337,6 +1334,9 @@ RivaScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
                case 16:	refreshArea = RivaRefreshArea16;	break;
                case 32:	refreshArea = RivaRefreshArea32;	break;
 	   }
+           xf86DisableRandR();
+           xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                      "Driver rotation enabled, RandR disabled\n");
 	}
 
 	ShadowFBInit(pScreen, refreshArea);
