@@ -1,28 +1,37 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis_setup.c,v 1.21 2003/10/30 18:53:43 twini Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis_setup.c,v 1.29 2004/01/23 22:29:05 twini Exp $ */
 /*
  * Basic hardware and memory detection
  *
- * Copyright 2001, 2002, 2003 by Thomas Winischhofer, Vienna, Austria.
+ * Copyright (C) 2001-2004 by Thomas Winischhofer, Vienna, Austria.
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holder not be used in
- * advertising or publicity pertaining to distribution of the software without
- * specific, written prior permission.  The copyright holder makes no representations
- * about the suitability of this software for any purpose.  It is provided
- * "as is" without express or implied warranty.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1) Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2) Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3) All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement: "This product includes
+ *    software developed by Thomas Winischhofer, Vienna, Austria."
+ * 4) The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
  *
- * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * Author:  Thomas Winischhofer <thomas@winischhofer.net>
+ * Author:  	Thomas Winischhofer <thomas@winischhofer.net>
+ *
+ * Ideas and methods for old series based on code by Can-Ru Yeou, SiS Inc.
  *
  */
  
@@ -110,7 +119,7 @@ sisOldSetup(ScrnInfoPtr pScrn)
     int     ramtype[4]  = { 5, 0, 1, 3 };
     int     config;
     int     temp, i;
-    unsigned char sr23, sr33, sr34, sr37;
+    unsigned char sr23, sr33, sr37;
 #if 0
     unsigned char newsr13, newsr28, newsr29;
 #endif
@@ -193,7 +202,6 @@ sisOldSetup(ScrnInfoPtr pScrn)
     if(pSiS->oldChipset >= OC_SIS82204) {
        inSISIDXREG(SISSR, 0x23, sr23);
        inSISIDXREG(SISSR, 0x33, sr33);
-       inSISIDXREG(SISSR, 0x34, sr34);
        if(pSiS->oldChipset >= OC_SIS530A) sr33 &= ~0x08;
        if(sr33 & 0x09) {   	  			/* 5597: Sync DRAM timing | One cycle EDO ram;   */
        		pSiS->Flags |= (sr33 & SYNCDRAM);	/* 6326: Enable SGRam timing | One cycle EDO ram */
@@ -267,7 +275,7 @@ sis300Setup(ScrnInfoPtr pScrn)
     unsigned int    config, pciconfig, sr3a, ramtype;
     unsigned char   temp;
     int		    cpubuswidth;
-    int 	    from = X_PROBED;
+    MessageType	    from = X_PROBED;
 
     pSiS->MemClock = SiSMclk(pSiS);
 
@@ -491,10 +499,11 @@ sis550Setup(ScrnInfoPtr pScrn)
 	  pciconfig = (pciReadByte(0x00000800, 0xcd) >> 1) & 0x03;
 	  if(pciconfig == 0x01)      pScrn->videoRam += 32768;
 	  else if(pciconfig == 0x03) pScrn->videoRam += 65536;
-	  
+
 	  if((pScrn->videoRam < 32768) || (pScrn->videoRam > 131072)) {
 	     xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-	     	"Illegal video Ram size (%d) detected, using BIOS setting\n");
+	     	"Illegal video Ram size (%d) detected, using BIOS setting\n",
+		pScrn->videoRam);
 	  } else {
 	     pSiS->BusWidth = 64;
 	     ramtype = 8;
@@ -593,7 +602,7 @@ sis550Setup(ScrnInfoPtr pScrn)
     if(!alldone) {
 
        if(pSiS->Chipset == PCI_CHIP_SIS660) {
-          inSISIDXREG(SISSR, 0x79, config);
+          inSISIDXREG(SISCR, 0x79, config);
 	  pScrn->videoRam = (1 << ((config & 0xf0) >> 4)) * 1024;
 	  pSiS->BusWidth = (config & 0x04) ? 128 : 64;
           ramtype = (config & 0x01) ? 8 : 4;
@@ -650,7 +659,7 @@ SiSSetup(ScrnInfoPtr pScrn)
 	break;
     case    PCI_CHIP_SIS550:
     case    PCI_CHIP_SIS650: /* + 740 */
-    case    PCI_CHIP_SIS660: /* + 661,741,660,760 */
+    case    PCI_CHIP_SIS660: /* + 661,741,760 */
         sis550Setup(pScrn);
 	break;
     case    PCI_CHIP_SIS5597:
