@@ -1,26 +1,33 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis_vb.c,v 1.29 2003/10/30 18:53:43 twini Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sis/sis_vb.c,v 1.41 2004/01/23 22:29:06 twini Exp $ */
 /*
  * Video bridge detection and configuration for 300, 315 and 330 series
  *
- * Copyright 2002, 2003 by Thomas Winischhofer, Vienna, Austria
+ * Copyright (C) 2001-2004 by Thomas Winischhofer, Vienna, Austria
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holder not be used in
- * advertising or publicity pertaining to distribution of the software without
- * specific, written prior permission.  The copyright holder makes no representations
- * about the suitability of this software for any purpose.  It is provided
- * "as is" without express or implied warranty.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1) Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2) Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3) All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement: "This product includes
+ *    software developed by Thomas Winischhofer, Vienna, Austria."
+ * 4) The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
  *
- * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESSED OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * Author: 	Thomas Winischhofer <thomas@winischhofer.net>
  *		
@@ -79,6 +86,26 @@ static const SiS_LCD_StStruct SiS315_LCD_Type[]=
 	{ VB_LCD_CUSTOM,      0,    0, LCD_CUSTOM,   }   /* f */
 };
 
+static const SiS_LCD_StStruct SiS661_LCD_Type[]=
+{
+        { VB_LCD_1024x768, 1024,  768, LCD_1024x768  },  /* 0 - invalid */
+	{ VB_LCD_800x600,   800,  600, LCD_800x600   },  /* 1 */
+	{ VB_LCD_1024x768, 1024,  768, LCD_1024x768  },  /* 2 */
+	{ VB_LCD_1280x1024,1280, 1024, LCD_1280x1024 },  /* 3 */
+	{ VB_LCD_640x480,   640,  480, LCD_640x480   },  /* 4 */
+	{ VB_LCD_1024x600, 1024,  600, LCD_1024x600  },  /* 5 */
+	{ VB_LCD_1152x864, 1152,  864, LCD_1152x864  },  /* 6 */
+	{ VB_LCD_1280x960, 1280,  960, LCD_1280x960  },  /* 7 */
+	{ VB_LCD_1152x768, 1152,  768, LCD_1152x768  },  /* 8 */
+	{ VB_LCD_1400x1050,1400, 1050, LCD_1400x1050 },  /* 9 */
+	{ VB_LCD_1280x768, 1280,  768, LCD_1280x768  },  /* a */
+	{ VB_LCD_1600x1200,1600, 1200, LCD_1600x1200 },  /* b */
+	{ VB_LCD_1280x800, 1280,  800, LCD_1280x800  },  /* c */
+	{ VB_LCD_1680x1050,1680, 1050, LCD_1680x1050 },  /* d */
+	{ VB_LCD_320x480,   320,  480, LCD_320x480   },  /* e */
+	{ VB_LCD_CUSTOM,      0,    0, LCD_CUSTOM,   }   /* f */
+};
+
 static Bool
 TestDDC1(ScrnInfoPtr pScrn)
 {
@@ -108,9 +135,9 @@ SiS_SISDetectCRT1(ScrnInfoPtr pScrn)
     if(SR1F & 0xc0) mustwait = TRUE;
 
     if(pSiS->VGAEngine == SIS_315_VGA) {
-       inSISIDXREG(SISCR,0x63,CR63);
+       inSISIDXREG(SISCR,pSiS->myCR63,CR63);
        CR63 &= 0x40;
-       andSISIDXREG(SISCR,0x63,0xBF);
+       andSISIDXREG(SISCR,pSiS->myCR63,0xBF);
     }
 
     inSISIDXREG(SISCR,0x17,CR17);
@@ -141,7 +168,7 @@ SiS_SISDetectCRT1(ScrnInfoPtr pScrn)
     }
 
     if(pSiS->VGAEngine == SIS_315_VGA) {
-       setSISIDXREG(SISCR,0x63,0xBF,CR63);
+       setSISIDXREG(SISCR,pSiS->myCR63,0xBF,CR63);
     }
 
     setSISIDXREG(SISCR,0x17,0x7F,CR17);
@@ -178,8 +205,6 @@ void SISCRT1PreInit(ScrnInfoPtr pScrn)
     }
 #endif
 
-    if(pSiS->ForceCRT1Type != CRT1_VGA) return;
-
     inSISIDXREG(SISCR, 0x32, CR32);
 
     if(CR32 & 0x20)  CRT1Detected = 1;
@@ -214,10 +239,9 @@ void SISLCDPreInit(ScrnInfoPtr pScrn)
     SISPtr  pSiS = SISPTR(pScrn);
     unsigned char CR32, CR36, CR37;
 
-    if(!(pSiS->VBFlags & VB_VIDEOBRIDGE)) {
-       pSiS->ForceCRT1Type = CRT1_VGA;
-       return;
-    }
+    pSiS->LCDwidth = 0;
+
+    if(!(pSiS->VBFlags & VB_VIDEOBRIDGE)) return;
 
     inSISIDXREG(SISCR, 0x32, CR32);
    
@@ -238,7 +262,7 @@ void SISLCDPreInit(ScrnInfoPtr pScrn)
      *
      * By default, CRT2 redetection is forced since 12/09/2003, as
      * I encountered numerous panels which deliver more or less
-     * bogus DDC data confusing the BIOS. Since out DDC detection
+     * bogus DDC data confusing the BIOS. Since our DDC detection
      * is waaaay better, we prefer it instead of the primitive
      * and buggy BIOS method.
      */
@@ -331,12 +355,18 @@ void SISLCDPreInit(ScrnInfoPtr pScrn)
 	        pSiS->LCDwidth = SiS300_LCD_Type[(CR36 & 0x0f)].LCDwidth;
                 pSiS->sishw_ext.ulCRT2LCDType = SiS300_LCD_Type[(CR36 & 0x0f)].LCDtype;
 	        if(CR37 & 0x10) pSiS->VBLCDFlags |= VB_LCD_EXPANDING;
-	     } else {
+	     } else if(pSiS->sishw_ext.jChipType < SIS_661) {
 	        pSiS->VBLCDFlags |= SiS315_LCD_Type[(CR36 & 0x0f)].VBLCD_lcdflag;
                 pSiS->LCDheight = SiS315_LCD_Type[(CR36 & 0x0f)].LCDheight;
 	        pSiS->LCDwidth = SiS315_LCD_Type[(CR36 & 0x0f)].LCDwidth;
                 pSiS->sishw_ext.ulCRT2LCDType = SiS315_LCD_Type[(CR36 & 0x0f)].LCDtype;
 	        if(CR37 & 0x10) pSiS->VBLCDFlags |= VB_LCD_EXPANDING;
+	     } else {
+		pSiS->VBLCDFlags |= SiS661_LCD_Type[(CR36 & 0x0f)].VBLCD_lcdflag;
+                pSiS->LCDheight = SiS661_LCD_Type[(CR36 & 0x0f)].LCDheight;
+	        pSiS->LCDwidth = SiS661_LCD_Type[(CR36 & 0x0f)].LCDwidth;
+                pSiS->sishw_ext.ulCRT2LCDType = SiS661_LCD_Type[(CR36 & 0x0f)].LCDtype;
+	        if(CR37 & 0x10) pSiS->VBLCDFlags |= VB_LCD_EXPANDING;
 	     }
 	     xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
 			"Detected LCD/Plasma panel (%dx%d, %d, %sexp., RGB%d [%02x%02x])\n",
@@ -351,22 +381,19 @@ void SISLCDPreInit(ScrnInfoPtr pScrn)
        }
     }
 
-    if(!(pSiS->VBFlags & CRT2_LCD)) {
-       pSiS->ForceCRT1Type = CRT1_VGA;
-    }
 }
 
 /* Detect CRT2-TV connector type and PAL/NTSC flag */
 void SISTVPreInit(ScrnInfoPtr pScrn)
 {
     SISPtr  pSiS = SISPTR(pScrn);
-    unsigned char SR16, SR38, CR32, CR35=0, CR38=0, CR79;
+    unsigned char SR16, SR38, CR32, CR35=0, CR38=0, CR79, CR39;
     int temp = 0;
 
-    if(!(pSiS->VBFlags & VB_VIDEOBRIDGE))
-        return;
+    if(!(pSiS->VBFlags & VB_VIDEOBRIDGE)) return;
 
     inSISIDXREG(SISCR, 0x32, CR32);
+    inSISIDXREG(SISCR, 0x35, CR35);
     inSISIDXREG(SISSR, 0x16, SR16);
     inSISIDXREG(SISSR, 0x38, SR38);
     switch(pSiS->VGAEngine) {
@@ -374,7 +401,7 @@ void SISTVPreInit(ScrnInfoPtr pScrn)
        if(pSiS->Chipset == PCI_CHIP_SIS630) temp = 0x35;
        break;
     case SIS_315_VGA:
-       if(pSiS->Chipset != PCI_CHIP_SIS660) temp = 0x38;
+       temp = 0x38;
        break;
     }
     if(temp) {
@@ -387,27 +414,64 @@ void SISTVPreInit(ScrnInfoPtr pScrn)
 	CR32, SR16, SR38);
 #endif
 
-    if(CR32 & 0x47)
-       pSiS->VBFlags |= CRT2_TV;
+    if(CR32 & 0x47) pSiS->VBFlags |= CRT2_TV;
 
-    if(CR32 & 0x04)
-       pSiS->VBFlags |= TV_SCART;
+    if(pSiS->SiS_SD_Flags & SiS_SD_SUPPORTYPBPR) {
+       if(CR32 & 0x80) pSiS->VBFlags |= CRT2_TV;
+    } else {
+       CR32 &= 0x7f;
+    }
+
+    if(CR32 & 0x01)
+       pSiS->VBFlags |= TV_AVIDEO;
     else if(CR32 & 0x02)
        pSiS->VBFlags |= TV_SVIDEO;
-    else if(CR32 & 0x01)
-       pSiS->VBFlags |= TV_AVIDEO;
-    else if(CR32 & 0x40)
-       pSiS->VBFlags |= (TV_SVIDEO | TV_HIVISION);
-    else if((CR38 & 0x04) && (pSiS->VBFlags & (VB_301LV | VB_302LV)))
-       pSiS->VBFlags |= TV_HIVISION_LV;
-    else if((CR38 & 0x04) && (pSiS->VBFlags & VB_CHRONTEL)) 
+    else if(CR32 & 0x04)
+       pSiS->VBFlags |= TV_SCART;
+    else if((CR32 & 0x40) && (pSiS->SiS_SD_Flags & SiS_SD_SUPPORTHIVISION))
+       pSiS->VBFlags |= (TV_HIVISION | TV_PAL);
+    else if((CR32 & 0x80) && (pSiS->SiS_SD_Flags & SiS_SD_SUPPORTYPBPR)) {
+       pSiS->VBFlags |= TV_YPBPR;
+       if(pSiS->Chipset == PCI_CHIP_SIS660) {
+          if(CR38 & 0x04) {
+             switch((CR35 & 0xE0)) {
+             case 0x20: pSiS->VBFlags |= TV_YPBPR525P; break;
+	     case 0x40: pSiS->VBFlags |= TV_YPBPR750P; break;
+	     case 0x60: pSiS->VBFlags |= TV_YPBPR1080I; break;
+	     default:   pSiS->VBFlags |= TV_YPBPR525I;
+	     }
+          }
+          inSISIDXREG(SISCR,0x39,CR39);
+	  CR39 &= 0x03;
+	  if(CR39 == 0x00)      pSiS->VBFlags |= TV_YPBPR43LB;
+	  else if(CR39 == 0x01) pSiS->VBFlags |= TV_YPBPR43;
+	  else if(CR39 == 0x02) pSiS->VBFlags |= TV_YPBPR169;
+	  else			pSiS->VBFlags |= TV_YPBPR43;
+       } else if(pSiS->SiS_SD_Flags & SiS_SD_SUPPORTYPBPR) {
+          if(CR38 & 0x08) {
+	     switch((CR38 & 0x30)) {
+	     case 0x10: pSiS->VBFlags |= TV_YPBPR525P; break;
+	     case 0x20: pSiS->VBFlags |= TV_YPBPR750P; break;
+	     case 0x30: pSiS->VBFlags |= TV_YPBPR1080I; break;
+	     default:   pSiS->VBFlags |= TV_YPBPR525I;
+	     }
+	  }
+	  if(pSiS->SiS_SD_Flags & SiS_SD_SUPPORTYPBPRAR) {
+             inSISIDXREG(SISCR,0x3B,CR39);
+	     CR39 &= 0x03;
+	     if(CR39 == 0x00)      pSiS->VBFlags |= TV_YPBPR43LB;
+	     else if(CR39 == 0x01) pSiS->VBFlags |= TV_YPBPR169;
+	     else if(CR39 == 0x03) pSiS->VBFlags |= TV_YPBPR43;
+	  }
+       }
+    } else if((CR38 & 0x04) && (pSiS->VBFlags & VB_CHRONTEL))
        pSiS->VBFlags |= (TV_CHSCART | TV_PAL);
     else if((CR38 & 0x08) && (pSiS->VBFlags & VB_CHRONTEL))
-       pSiS->VBFlags |= (TV_CHHDTV | TV_NTSC);
-	        
-    if(pSiS->VBFlags & (TV_SCART | TV_SVIDEO | TV_AVIDEO | TV_HIVISION | TV_HIVISION_LV)) {
+       pSiS->VBFlags |= (TV_CHYPBPR525I | TV_NTSC);
+
+    if(pSiS->VBFlags & (TV_SCART | TV_SVIDEO | TV_AVIDEO)) {
        if(pSiS->VGAEngine == SIS_300_VGA) {
-	  /* TW: Should be SR38, but this does not work. */
+	  /* Should be SR38, but this does not work. */
 	  if(SR16 & 0x20)
 	     pSiS->VBFlags |= TV_PAL;
           else
@@ -433,14 +497,14 @@ void SISTVPreInit(ScrnInfoPtr pScrn)
  	  } else
 	     pSiS->VBFlags |= TV_NTSC;
        } else if(pSiS->Chipset == PCI_CHIP_SIS660) {
-          inSISIDXREG(SISCR, 0x35, CR35);
           if(SR38 & 0x01) {
 	     pSiS->VBFlags |= TV_PAL;
 	     if(CR35 & 0x04)      pSiS->VBFlags |= TV_PALM;
 	     else if(CR35 & 0x08) pSiS->VBFlags |= TV_PALN;
-	  } else
+	  } else {
 	     pSiS->VBFlags |= TV_NTSC;
 	     if(CR35 & 0x02)      pSiS->VBFlags |= TV_NTSCJ;
+	  }
        } else {	/* 315, 330 */
 	  if(SR38 & 0x01) {
              pSiS->VBFlags |= TV_PAL;
@@ -450,16 +514,29 @@ void SISTVPreInit(ScrnInfoPtr pScrn)
 	     pSiS->VBFlags |= TV_NTSC;
        }
     }
-    
-    if(pSiS->VBFlags & (TV_SCART | TV_SVIDEO | TV_AVIDEO | TV_HIVISION | TV_HIVISION_LV | TV_CHSCART | TV_CHHDTV)) {
-       xf86DrvMsg(pScrn->scrnIndex, X_PROBED,
-			"%sTV standard %s\n",
-			(pSiS->VBFlags & (TV_CHSCART | TV_CHHDTV)) ? "Using " : "Detected default ",
-			(pSiS->VBFlags & TV_NTSC) ?
-			   ((pSiS->VBFlags & TV_CHHDTV) ? "480i HDTV" :
-			      ((pSiS->VBFlags & TV_NTSCJ) ? "NTSCJ" : "NTSC")) :
-			   ((pSiS->VBFlags & TV_PALM) ? "PALM" :
-			   	((pSiS->VBFlags & TV_PALN) ? "PALN" : "PAL")) );
+
+    if(pSiS->VBFlags & (TV_SCART|TV_SVIDEO|TV_AVIDEO)) {
+       xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Detected default TV standard %s\n",
+          (pSiS->VBFlags & TV_NTSC) ?
+	     ((pSiS->VBFlags & TV_NTSCJ) ? "NTSCJ" : "NTSC") :
+	         ((pSiS->VBFlags & TV_PALM) ? "PALM" :
+		     ((pSiS->VBFlags & TV_PALN) ? "PALN" : "PAL")));
+    }
+
+    if(pSiS->VBFlags & TV_HIVISION) {
+       xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "BIOS reports HiVision TV\n");
+    }
+
+    if((pSiS->VBFlags & VB_CHRONTEL) && (pSiS->VBFlags & (TV_CHSCART|TV_CHYPBPR525I))) {
+       xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Chrontel: %s forced\n",
+       	(pSiS->VBFlags & TV_CHSCART) ? "SCART (PAL)" : "YPbPr (480i)");
+    }
+
+    if(pSiS->VBFlags & TV_YPBPR) {
+       xf86DrvMsg(pScrn->scrnIndex, X_PROBED, "Detected YPbPr TV (%s)\n",
+         (pSiS->VBFlags & TV_YPBPR525I) ? "480i" :
+	     ((pSiS->VBFlags & TV_YPBPR525P) ? "480p" :
+	        ((pSiS->VBFlags & TV_YPBPR750P) ? "720p" : "1080i")));
     }
 }
 
@@ -469,11 +546,10 @@ void SISCRT2PreInit(ScrnInfoPtr pScrn)
     SISPtr  pSiS = SISPTR(pScrn);
     unsigned char CR32; 
 
-    if(!(pSiS->VBFlags & VB_VIDEOBRIDGE))
-       return;
+    if(!(pSiS->VBFlags & VB_VIDEOBRIDGE))  return;
 
     /* CRT2-VGA not supported on LVDS and 30xLV */
-    if(pSiS->VBFlags & (VB_LVDS|VB_301LV|VB_302LV))
+    if(pSiS->VBFlags & (VB_LVDS|VB_301LV|VB_302LV|VB_302ELV))
        return;
 
     inSISIDXREG(SISCR, 0x32, CR32);
@@ -520,8 +596,9 @@ void SISCRT2PreInit(ScrnInfoPtr pScrn)
              }
           }
        }
-
+#ifdef SISDUALHEAD
     }
+#endif    
 }
 
 
