@@ -1,7 +1,7 @@
 #!/bin/sh
 
 #
-# $XFree86: xc/programs/Xserver/hw/xfree86/etc/Xinstall.sh,v 1.58 2003/10/19 03:02:19 dawes Exp $
+# $XFree86: xc/programs/Xserver/hw/xfree86/etc/Xinstall.sh,v 1.68 2004/02/15 18:38:32 dawes Exp $
 #
 # Copyright  2000 by Precision Insight, Inc.
 # Copyright  2000, 2001 by VA Linux Systems, Inc.
@@ -45,11 +45,11 @@
 # Fallbacks for when the bindist version can't be auto-detected.
 # These should be updated for each release.
 
-SNAPSHOT=n
+SNAPSHOT=y
 
 if [ $SNAPSHOT = y ]; then
 	FULLPREFIX=XXX
-	VERSION=4.3.99.15
+	VERSION=4.3.99.903
 	PATCHLEVEL=0
 	FULLVERSION=$VERSION
 else
@@ -106,10 +106,13 @@ RUNDIR=$ROOTDIR/usr/X11R6
 ETCDIR=$ROOTDIR/etc
 VARDIR=$ROOTDIR/var
 
-OLDFILES=""
+OLDFILES=" \
+	$RUNDIR/include/freetype2/ft2build.h \
+	"
 
 OLDDIRS=" \
 	$RUNDIR/lib/X11/xkb/compiled \
+	$RUNDIR/lib/X11/xkb/geometry/hp \
 	"
 
 OLDMODULES=" \
@@ -153,8 +156,6 @@ OPTDIST=" \
 	Xvfb.tgz \
 	Xf100.tgz \
 	Xfcyr.tgz \
-	Xflat2.tgz \
-	Xfnon.tgz \
 	Xfscl.tgz \
 	Xhtml.tgz \
 	Xjdoc.tgz \
@@ -267,14 +268,12 @@ Description()
 		echo "X print server";;
 	Xvfb*)
 		echo "Virtual framebuffer X server";;
+	Xdrm*)
+		echo "DRM kernel module source";;
 	Xf100*)
 		echo "100dpi fonts";;
 	Xfcyr*)
 		echo "Cyrillic fonts";;
-	Xflat2*)
-		echo "Latin-2 fonts";;
-	Xfnon*)
-		echo "Some large fonts";;
 	Xfscl*)
 		echo "Scaled fonts (Speedo, Type1 and TTF)";;
 	Xhtml*)
@@ -507,7 +506,7 @@ FindDistName()
 	CYGWIN*)
 		case "$OsArch" in
 		i*86)
-			DistName="Cygwin-ix86"
+			DistName="Cygwin"
 			;;
 		*)
 			Message="Cygwin binaries are only available for ix86 platforms"
@@ -524,11 +523,21 @@ FindDistName()
 			1.4* | 5.*)
 				DistName="Darwin-ppc-5.x"
 				;;
-			6.*)
+			6.[0-2]*)
+				if [ -d /System/Library/Frameworks/ApplicationServices.framework ]; then
+					Message="No binaries available for Mac OS X 10.2.0 - 10.2.2"
+				else
+					DistName="Darwin-ppc-6.x"
+				fi
+				;;
+			6.[3-9]*)
 				DistName="Darwin-ppc-6.x"
 				;;
-			[7-9].*)
-				Message="No Darwin/ppc binaries available for this OS version. Try Darwin-ppc-6.x"
+			7.*)
+				DistName="Darwin-ppc-7.x"
+				;;
+			[8-9].*)
+				Message="No Darwin/ppc binaries available for this OS version.  Try Darwin-ppc-7.x"
 				;;
 			*)
 				Message="No Darwin/ppc binaries available for this OS version"
@@ -540,11 +549,11 @@ FindDistName()
 			1.4* | 5.*)
 				DistName="Darwin-ix86-5.x"
 				;;
-			6.*)
-				DistName="Darwin-ix86-6.x"
+			6.* | 7.*)
+				DistName="Darwin-ix86-7.x"
 				;;
-			[7-9].*)
-				Message="No Darwin/ix86 binaries available for this OS version. Try Darwin-ix86-6.x"
+			[8-9].*)
+				Message="No Darwin/ix86 binaries available for this OS version.  Try Darwin-ix86-7.x"
 				;;
 			*)
 				Message="No Darwin/ix86 binaries available for this OS version"
@@ -744,8 +753,8 @@ FindDistName()
 			5.8*)
 				DistName="Solaris-8"
 				;;
-			*)
-				Message="No Solaris/x86 binaries available for this version"
+			5.9*)
+				Message="No Solaris/x86 binaries available for this version.  Try Solaris-8."
 				;;
 			esac
 			;;
@@ -1024,9 +1033,17 @@ Darwin)
             SERVDIST="Xxserv.tgz"
         fi
 	;;
-FreeBSD|OpenBSD)
+FreeBSD)
 	VARDIST="Xvar.tgz"
 	XKBDBDIR="$VARDIR/db/xkb"
+	EXTRAOPTDIST="Xdrm.tgz"
+	;;
+OpenBSD)
+	VARDIST="Xvar.tgz"
+	XKBDBDIR="$VARDIR/db/xkb"
+	;;
+NetBSD)
+	EXTRAOPTDIST="Xdrm.tgz"
 	;;
 Interactive)	# Need the correct name for this
 	EXTRADIST="Xbin1.tgz"
@@ -1035,6 +1052,7 @@ Interactive)	# Need the correct name for this
 Linux)
 	VARDIST="Xvar.tgz"
 	XKBDBDIR="$VARDIR/lib/xkb"
+	EXTRAOPTDIST="Xdrm.tgz"
 	;;
 esac
 
@@ -1743,6 +1761,68 @@ if [ -d $RUNDIR/lib/modules ]; then
 	fi
 fi
 
+case "$OsName" in
+FreeBSD|NetBSD)
+	DRMBUILDDIR="$RUNDIR/src/bsd/drm/kernel"
+	;;
+Linux)
+	DRMBUILDDIR="$RUNDIR/src/linux/drm/kernel"
+	;;
+esac
+
+DoDrmBuild=
+
+if [ -f $DRMBUILDDIR/Makefile ]; then
+	echo ""
+	echo "If you have source for your current kernel installed, you can"
+	echo "have new DRM kernel modules built and installed.  Note: this will"
+	echo "overwrite any existing DRM kernel modules you may have installed."
+	echo "If you'd prefer to save them before installing new ones, answer 'n'"
+	echo "here and follow the build/install instructions manually later."
+	echo ""
+	Echo "Do you want to build and install new DRM kernel modules? (y/n) [n] "
+	read response
+	case "$response" in
+	[yY]*)
+		DoDrmBuild=1
+		;;
+	*)
+		echo ""
+		echo "To build the DRM modules manually, run 'make' from the"
+		echo "$DRMBUILDDIR directory.  Once built, install"
+		echo "them by running 'make install' from the same directory."
+		;;
+	esac
+fi
+
+if [ X"$DoDrmBuild" != X ]; then
+	cd $DRMBUILDDIR && \
+		echo "" && \
+		echo "Building DRM modules" && \
+		make clean > build.log 2>&1 && \
+		make >> build.log 2>&1
+	if [ $? != 0 ]; then
+		echo ""
+		echo "DRM module build failed."
+		echo "See $DRMBUILDDIR/build.log for details."
+	else
+		echo ""
+		echo "Installing DRM modules"
+		make install > install.log 2>&1
+		if [ $? != 0 ]; then
+			echo ""
+			echo "DRM module install failed."
+			echo "See $DRMBUILDDIR/install.log for details."
+		else
+			echo ""
+			echo "DRM module build and install was successful."
+			echo "You may need to manually unload the old module and reload"
+			echo "the new ones, or reboot to have this done automatically."
+		fi
+	fi
+	cd $WDIR
+fi
+
 echo ""
 echo "Installation complete."
 
