@@ -2,10 +2,10 @@
  *	$Xorg: input.c,v 1.3 2000/08/17 19:55:08 cpqbld Exp $
  */
 
-/* $XFree86: xc/programs/xterm/input.c,v 3.66 2003/10/21 23:16:19 dawes Exp $ */
+/* $XFree86: xc/programs/xterm/input.c,v 3.69 2003/12/31 17:12:28 dickey Exp $ */
 
 /*
- * Copyright 1999-2001,2002 by Thomas E. Dickey
+ * Copyright 1999-2002,2003 by Thomas E. Dickey
  *
  *                         All Rights Reserved
  *
@@ -468,7 +468,7 @@ Input(TKeyboard * keyboard,
 	&& keyboard->type != keyboardIsVT220
 #endif
 #if OPT_VT52_MODE
-	&& screen->ansi_level != 0
+	&& screen->vtXX_level != 0
 #endif
 	) {
 /*
@@ -569,7 +569,7 @@ Input(TKeyboard * keyboard,
 	key = TRUE;
 #if 0				/* OPT_SUNPC_KBD should suppress - but only for vt220 compatibility */
     } else if (keyboard->type == keyboardIsVT220
-	       && screen->ansi_level <= 1
+	       && screen->vtXX_level <= 1
 	       && IsEditFunctionKey(keysym)) {
 	key = FALSE;		/* ignore editing-keypad in vt100 mode */
 #endif
@@ -686,18 +686,34 @@ Input(TKeyboard * keyboard,
 	     */
 	    if (screen->meta_sends_esc
 		&& ((event->state & term->misc.meta_left) != 0
-		    || (event->state & term->misc.meta_right)) != 0) {
+		    || (event->state & term->misc.meta_right) != 0)) {
 		TRACE(("...input-char is modified by META\n"));
 		eightbit = False;
 		unparseputc(ESC, pty);	/* escape */
 	    }
 #endif
 	    if (eightbit && screen->input_eight_bits) {
-		if (CharOf(*string) < 128) {
-		    TRACE(("...input shift from %d to %d\n",
-			   CharOf(*string),
-			   CharOf(*string) | 0x80));
+		IChar ch = CharOf(*string);
+		if (ch < 128) {
 		    *string |= 0x80;
+		    TRACE(("...input shift from %d to %d (%#x to %#x)\n",
+			   ch, CharOf(*string),
+			   ch, CharOf(*string)));
+#if OPT_WIDE_CHARS
+		    if (screen->utf8_mode) {
+			/*
+			 * We could interpret the incoming code as "in the
+			 * current locale", but it's simpler to treat it as
+			 * a Unicode value to translate to UTF-8.
+			 */
+			ch = CharOf(*string);
+			nbytes = 2;
+			string[0] = 0xc0 | ((ch >> 6) & 0x3);
+			string[1] = 0x80 | (ch & 0x3f);
+			TRACE(("...encoded %#x in UTF-8 as %#x,%#x\n",
+			       ch, string[0], string[1]));
+		    }
+#endif
 		}
 		eightbit = False;
 	    }
@@ -1048,7 +1064,7 @@ TranslationsUseKeyword(Widget w, const char *keyword)
     return result;
 }
 
-#define SaveMask(name)	term->misc.name = mask;\
+#define SaveMask(name)	term->misc.name |= mask;\
 			TRACE(("%s mask %#lx is%s modifier\n", \
 				#name, \
 				term->misc.name, \
@@ -1068,15 +1084,26 @@ VTInitModifiers(void)
     Display *dpy = XtDisplay(term);
     XModifierKeymap *keymap = XGetModifierMapping(dpy);
     unsigned long mask;
+    int min_keycode, max_keycode, keysyms_per_keycode = 0;
 
     if (keymap != 0) {
 
 	TRACE(("VTInitModifiers\n"));
+
+	XDisplayKeycodes(dpy, &min_keycode, &max_keycode);
+	XGetKeyboardMapping(dpy, min_keycode, (max_keycode - min_keycode + 1),
+			    &keysyms_per_keycode);
+
 	for (i = k = 0, mask = 1; i < 8; i++, mask <<= 1) {
 	    for (j = 0; j < keymap->max_keypermod; j++) {
 		KeyCode code = keymap->modifiermap[k];
 		if (code != 0) {
-		    KeySym keysym = XKeycodeToKeysym(dpy, code, 0);
+		    KeySym keysym;
+		    int l = 0;
+		    do {
+			keysym = XKeycodeToKeysym(dpy, code, l);
+			l++;
+		    } while (!keysym && l < keysyms_per_keycode);
 		    if (keysym == XK_Num_Lock) {
 			SaveMask(num_lock);
 		    } else if (keysym == XK_Alt_L) {
